@@ -1,25 +1,34 @@
 """Docstring violation definition."""
+
 from itertools import dropwhile
 from functools import partial
 from collections import namedtuple
+from typing import Iterable, Optional, List, Callable, Any
 
 from .utils import is_blank
+from .parser import Definition
 
 
-__all__ = ('Error', 'ErrorRegistry')
+__all__ = ('Error', 'ErrorRegistry', 'conventions')
 
 
 ErrorParams = namedtuple('ErrorParams', ['code', 'short_desc', 'context'])
 
 
-class Error(object):
+class Error:
     """Error in docstring style."""
 
     # Options that define how errors are printed:
     explain = False
     source = False
 
-    def __init__(self, code, short_desc, context, *parameters):
+    def __init__(
+            self,
+            code: str,
+            short_desc: str,
+            context: str,
+            *parameters: Iterable[str]
+    ) -> None:
         """Initialize the object.
 
         `parameters` are specific to the created error.
@@ -29,10 +38,10 @@ class Error(object):
         self.short_desc = short_desc
         self.context = context
         self.parameters = parameters
-        self.definition = None
-        self.explanation = None
+        self.definition = None  # type: Optional[Definition]
+        self.explanation = None  # type: Optional[str]
 
-    def set_context(self, definition, explanation):
+    def set_context(self, definition: Definition, explanation: str) -> None:
         """Set the source code context for this error."""
         self.definition = definition
         self.explanation = explanation
@@ -41,7 +50,7 @@ class Error(object):
     line = property(lambda self: self.definition.error_lineno)
 
     @property
-    def message(self):
+    def message(self) -> str:
         """Return the message to print to the user."""
         ret = '{}: {}'.format(self.code, self.short_desc)
         if self.context is not None:
@@ -50,11 +59,13 @@ class Error(object):
         return ret
 
     @property
-    def lines(self):
+    def lines(self) -> str:
         """Return the source code lines for this error."""
+        if self.definition is None:
+            return ''
         source = ''
         lines = self.definition.source
-        offset = self.definition.start
+        offset = self.definition.start  # type: ignore
         lines_stripped = list(reversed(list(dropwhile(is_blank,
                                                       reversed(lines)))))
         numbers_width = len(str(offset + len(lines_stripped)))
@@ -68,9 +79,10 @@ class Error(object):
                 break
         return source
 
-    def __str__(self):
-        self.explanation = '\n'.join(l for l in self.explanation.split('\n')
-                                     if not is_blank(l))
+    def __str__(self) -> str:
+        if self.explanation:
+            self.explanation = '\n'.join(l for l in self.explanation.split('\n')
+                                         if not is_blank(l))
         template = '{filename}:{line} {definition}:\n        {message}'
         if self.source and self.explain:
             template += '\n\n{explanation}\n\n{lines}\n'
@@ -78,26 +90,26 @@ class Error(object):
             template += '\n\n{lines}\n'
         elif self.explain and not self.source:
             template += '\n\n{explanation}\n\n'
-        return template.format(**dict((name, getattr(self, name)) for name in
+        return template.format(**{name: getattr(self, name) for name in
                                ['filename', 'line', 'definition', 'message',
-                                'explanation', 'lines']))
+                                'explanation', 'lines']})
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return str(self)
 
-    def __lt__(self, other):
+    def __lt__(self, other: 'Error') -> bool:
         return (self.filename, self.line) < (other.filename, other.line)
 
 
-class ErrorRegistry(object):
+class ErrorRegistry:
     """A registry of all error codes, divided to groups."""
 
-    groups = []
+    groups = []  # type: ignore
 
-    class ErrorGroup(object):
+    class ErrorGroup:
         """A group of similarly themed errors."""
 
-        def __init__(self, prefix, name):
+        def __init__(self, prefix: str, name: str) -> None:
             """Initialize the object.
 
             `Prefix` should be the common prefix for errors in this group,
@@ -107,9 +119,14 @@ class ErrorRegistry(object):
             """
             self.prefix = prefix
             self.name = name
-            self.errors = []
+            self.errors = []  # type: List[ErrorParams]
 
-        def create_error(self, error_code, error_desc, error_context=None):
+        def create_error(
+                self,
+                error_code: str,
+                error_desc: str,
+                error_context: Optional[str]=None,
+        ) -> Callable[[Iterable[str]], Error]:
             """Create an error, register it to this group and return it."""
             # TODO: check prefix
 
@@ -119,21 +136,21 @@ class ErrorRegistry(object):
             return factory
 
     @classmethod
-    def create_group(cls, prefix, name):
+    def create_group(cls, prefix: str, name: str) -> ErrorGroup:
         """Create a new error group and return it."""
         group = cls.ErrorGroup(prefix, name)
         cls.groups.append(group)
         return group
 
     @classmethod
-    def get_error_codes(cls):
+    def get_error_codes(cls) -> Iterable[str]:
         """Yield all registered codes."""
         for group in cls.groups:
             for error in group.errors:
                 yield error.code
 
     @classmethod
-    def to_rst(cls):
+    def to_rst(cls) -> str:
         """Output the registry as reStructuredText, for documentation."""
         sep_line = '+' + 6 * '-' + '+' + '-' * 71 + '+\n'
         blank_line = '|' + 78 * ' ' + '|\n'
@@ -230,17 +247,27 @@ D412 = D4xx.create_error('D412', 'No blank lines allowed between a section '
 D413 = D4xx.create_error('D413', 'Missing blank line after last section',
                          '{0!r}')
 D414 = D4xx.create_error('D414', 'Section has no content', '{0!r}')
-
+D415 = D4xx.create_error('D415', 'First line should end with a period, question '
+                                 'mark, or exclamation point', 'not {0!r}')
+D416 = D4xx.create_error('D416', 'Section name should end with a semicolon',
+                         '{0!r}, not {1!r}')
+D417 = D4xx.create_error('D417', 'Missing arguments in the function docstring',
+                         'argument(s) {0!r} missing in function {1!r} docstring')
 
 class AttrDict(dict):
-    def __getattr__(self, item):
+    def __getattr__(self, item: str) -> Any:
         return self[item]
 
+
 all_errors = set(ErrorRegistry.get_error_codes())
 
+
 conventions = AttrDict({
     'pep257': all_errors - {'D203', 'D212', 'D213', 'D214', 'D215', 'D404',
                             'D405', 'D406', 'D407', 'D408', 'D409', 'D410',
-                            'D411'},
-    'numpy': all_errors - {'D107', 'D203', 'D212', 'D213', 'D402', 'D413'}
+                            'D411', 'D415', 'D416', 'D417'},
+    'numpy': all_errors - {'D107', 'D203', 'D212', 'D213', 'D402', 'D413',
+                           'D415', 'D416', 'D417'},
+    'google': all_errors - {'D203', 'D204', 'D213', 'D215', 'D400', 'D401',
+                            'D404', 'D406', 'D407', 'D408', 'D409'}
 })
